@@ -2,8 +2,7 @@
 #define GSID_H
 
 #define NUMSIDREGS 0x19
-#define SIDWRITEDELAY 9 // lda $xxxx,x 4 cycles, sta $d400,x 5 cycles
-#define SIDWAVEDELAY 4 // and $xxxx,x 4 cycles extra
+#define SIDWRITEDELAY 14 // lda $xxxx,x 4 cycles, sta $d400,x 5 cycles, dex 2 cycles, bpl 3 cycles
 
 typedef struct
 {
