@@ -30,5 +30,7 @@
 
 int sound_init(unsigned b, unsigned mr, unsigned writer, unsigned hardsid, unsigned m, unsigned ntsc, unsigned multiplier, unsigned catweasel, unsigned interpolate, unsigned customclockrate);
 void sound_uninit(void);
+void sound_suspend(void);
+void sound_flush(void);
 
 #endif
